from selene import conditions

visible = conditions.visible
in_dom = conditions.in_dom
enabled = conditions.enabled
empty = conditions.empty
existing = conditions.exist
clickable = conditions.clickable
hidden = conditions.hidden
blank = conditions.blank
or_not_to_be = conditions.or_not_to_be
not_ = conditions.not_
