from selene import bys

css = bys.by_css
xpath = bys.by_xpath
name = bys.by_name
link_text = bys.by_link_text
partial_link_text = bys.by_partial_link_text

text = bys.by_text
partial_text = bys.with_text


be_following_sibling = bys.following_sibling
be_parent = bys.parent
be_first_child = bys.first_child
